import re
import json
import itertools
from dataclasses import dataclass
from typing_extensions import override
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    Tuple,
    Union,
    Literal,
    Iterable,
    Optional,
    TypedDict,
)

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .models import Mention


class MessageSegment(BaseMessageSegment["Message"]):
    """
    飞书 协议 MessageSegment 适配。具体方法参考协议消息段类型或源码。
    """

    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @override
    def is_text(self) -> bool:
        return self.type == "text"

    @override
    def __str__(self) -> str:
        if self.is_text():
            return self.data.get("text", "")
        return ""

    @override
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return Message(self) + (
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @override
    def __radd__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return (
            MessageSegment.text(other) if isinstance(other, str) else Message(other)
        ) + self

    @staticmethod
    def text(text: str) -> "Text":
        return Text("text", {"text": str(text)})

    @staticmethod
    def at(user_id: str):
        return At("at", {"user_id": user_id})

    @staticmethod
    def post(
        title: str, content: List[List["PostMessageNode"]], language: str = "zh_cn"
    ) -> "Post":
        return Post("post", data={language: {"title": title, "content": content}})

    @staticmethod
    def image(image_key: str) -> "Image":
        return Image("image", {"image_key": image_key})

    @staticmethod
    def interactive(
        header: "InteractiveHeader",
        config: "InteractiveConfig",
        elements: Optional[List[Dict[str, Any]]] = None,
        i18n_elements: Optional[List[Dict[str, Any]]] = None,
    ):
        elements_key = "elements" if elements else "i18n_elements"
        elements_value = elements or i18n_elements

        return Interactive(
            "interactive",
            {
                "header": header,
                elements_key: elements_value,
                "config": config,
            },
        )

    @staticmethod
    def interactive_template(
        template_id: str, template_variable: Dict[str, Any]
    ) -> "InteractiveTemplate":
        return InteractiveTemplate(
            "interactive",
            {
                "template_id": template_id,
                "template_variable": template_variable,
            },
        )

    @staticmethod
    def todo(task_id: str, summary: "_PostData", due_time: str):
        return Todo(
            "todo", {"task_id": task_id, "summary": summary, "due_time": due_time}
        )

    @staticmethod
    def hongbao(text: str) -> "Hongbao":
        return Hongbao("hongbao", {"text": text})

    @staticmethod
    def system(template: str, from_user: List[str], to_chatters: List[str]) -> "System":
        return System(
            "system",
            {"template": template, "from_user": from_user, "to_chatters": to_chatters},
        )

    @staticmethod
    def location(name: str, longitude: str, latitude: str) -> "Location":
        return Location(
            "location",
            {"name": name, "longitude": longitude, "latitude": latitude},
        )

    @staticmethod
    def video_chat(topic: str, start_time: str):
        return VideoChat(
            "video_chat",
            {"topic": topic, "start_time": start_time},
        )

    @staticmethod
    def share_chat(chat_id: str) -> "MessageSegment":
        return ShareChat("share_chat", {"chat_id": chat_id})

    @staticmethod
    def share_user(user_id: str) -> "MessageSegment":
        return ShareUser("share_user", {"user_id": user_id})

    @staticmethod
    def audio(file_key: str, duration: Optional[int] = None) -> "MessageSegment":
        return Audio("audio", {"file_key": file_key, "duration": duration})

    @staticmethod
    def media(
        file_key: str,
        image_key: Optional[str] = None,
        file_name: Optional[str] = None,
        duration: Optional[int] = None,
    ) -> "MessageSegment":
        return Media(
            "media",
            {
                "file_key": file_key,
                "image_key": image_key,
                "file_name": file_name,
                "duration": duration,
            },
        )

    @staticmethod
    def file(file_key: str, file_name: Optional[str] = None) -> "MessageSegment":
        return File("file", {"file_key": file_key, "file_name": file_name})

    @staticmethod
    def folder(file_key: str, file_name: str) -> "MessageSegment":
        return Folder("folder", {"file_key": file_key, "file_name": file_name})

    @staticmethod
    def sticker(file_key: str) -> "MessageSegment":
        return Sticker("sticker", {"file_key": file_key})

    def to_post(self):
        if self.type == "image":
            self.type = "img"


class _TextData(TypedDict):
    text: str


@dataclass
class Text(MessageSegment):
    if TYPE_CHECKING:
        data: _TextData

    @override
    def __str__(self) -> str:
        return self.data["text"]


class _AtData(TypedDict):
    user_id: str


@dataclass
class At(MessageSegment):
    if TYPE_CHECKING:
        data: _AtData

    @override
    def __str__(self) -> str:
        return f"@{self.data['user_id']}"


class _ImageData(TypedDict):
    image_key: str


@dataclass
class Image(MessageSegment):
    if TYPE_CHECKING:
        data: _ImageData

    @override
    def __str__(self) -> str:
        return f"[image:{self.data['image_key']!r}]"


class InteractiveHeaderTitle(TypedDict):
    tag: Literal["plain_text"]
    content: Optional[str]
    i18n: Optional[Dict[str, str]]
    template: Optional[str]


class InteractiveHeader(TypedDict):
    title: InteractiveHeaderTitle


class InteractiveConfig(TypedDict):
    enable_forward: Optional[bool]
    update_multi: Optional[bool]


class _InteractiveData(TypedDict):
    header: InteractiveHeader
    elements: Optional[List[Dict[str, Any]]]
    i18n_elements: Optional[List[Dict[str, Any]]]
    config: InteractiveConfig


class Interactive(MessageSegment):
    if TYPE_CHECKING:
        data: _InteractiveData

    @override
    def __str__(self) -> str:
        return f"[interactive:{self.data!r}]"


class _InteractiveTemplateData(TypedDict):
    template_id: str
    template_variable: Dict[str, Any]


@dataclass
class InteractiveTemplate(MessageSegment):
    if TYPE_CHECKING:
        data: _InteractiveTemplateData

    @override
    def __str__(self) -> str:
        return f"[interactive_template:{self.data!r}]"


class _ShareChatData(TypedDict):
    chat_id: str


@dataclass
class ShareChat(MessageSegment):
    if TYPE_CHECKING:
        data: _ShareChatData

    @override
    def __str__(self) -> str:
        return f"[share_chat:{self.data['chat_id']!r}]"


class _ShareUserData(TypedDict):
    user_id: str


@dataclass
class ShareUser(MessageSegment):
    if TYPE_CHECKING:
        data: _ShareUserData

    @override
    def __str__(self) -> str:
        return f"[share_user:{self.data['user_id']!r}]"


class _AudioData(TypedDict):
    file_key: str
    duration: Optional[int]


@dataclass
class Audio(MessageSegment):
    if TYPE_CHECKING:
        data: _AudioData

    @override
    def __str__(self) -> str:
        return f"[audio:{self.data!r}]"


class _MediaData(TypedDict):
    file_key: str
    image_key: Optional[str]
    file_name: Optional[str]
    duration: Optional[int]


@dataclass
class Media(MessageSegment):
    if TYPE_CHECKING:
        data: _MediaData

    @override
    def __str__(self) -> str:
        return f"[media:{self.data['file_key']!r}]"


class _FileData(TypedDict):
    file_key: str
    file_name: Optional[str]


@dataclass
class File(MessageSegment):
    if TYPE_CHECKING:
        data: _FileData

    @override
    def __str__(self) -> str:
        return f"[file:{self.data!r}]"


class _FolderData(TypedDict):
    file_key: str
    file_name: Optional[str]


@dataclass
class Folder(MessageSegment):
    if TYPE_CHECKING:
        data: _FolderData

    @override
    def __str__(self) -> str:
        return f"[folder:{self.data!r}]"


class _StickerData(TypedDict):
    file_key: str


@dataclass
class Sticker(MessageSegment):
    if TYPE_CHECKING:
        data: _StickerData

    @override
    def __str__(self) -> str:
        return f"[sticker:{self.data['file_key']!r}]"


class PostMessageNode(TypedDict):
    tag: str


class PostMessageNodeStylable(TypedDict):
    style: Optional[List[Literal["bold", "underline", "lineThrough", "italic"]]]


class PostText(PostMessageNode, PostMessageNodeStylable):
    text: str
    un_escape: Optional[bool]


class PostA(PostMessageNode, PostMessageNodeStylable):
    text: str
    href: str


class PostAt(PostMessageNode, PostMessageNodeStylable):
    user_id: str
    user_name: Optional[str]


class PostImg(PostMessageNode):
    image_key: str


class PostMedia(PostMessageNode):
    file_key: str
    image_key: Optional[str]


class PostEmotion(PostMessageNode):
    emoji_type: str


class _PostData(TypedDict):
    title: str
    content: List[List[PostMessageNode]]


@dataclass
class Post(MessageSegment):
    if TYPE_CHECKING:
        data: Dict[str, _PostData]  # i18n

    @override
    def __str__(self) -> str:
        return f"[post:{self.data!r}]"


class _SystemData(TypedDict):
    template: str
    from_user: List[str]
    to_chatters: List[str]


@dataclass
class System(MessageSegment):
    if TYPE_CHECKING:
        data: _SystemData

    @override
    def __str__(self) -> str:
        return f"[system:{self.data!r}]"


class _LocationData(TypedDict):
    name: str
    longitude: str
    latitude: str


@dataclass
class Location(MessageSegment):
    if TYPE_CHECKING:
        data: _LocationData

    @override
    def __str__(self) -> str:
        return f"[location:{self.data!r}]"


class _VideoChatData(TypedDict):
    topic: str
    start_time: str


@dataclass
class VideoChat(MessageSegment):
    if TYPE_CHECKING:
        data: _VideoChatData

    @override
    def __str__(self) -> str:
        return f"[video_chat:{self.data!r}]"


class _TodoData(TypedDict):
    task_id: str
    summary: _PostData
    due_time: str


@dataclass
class Todo(MessageSegment):
    if TYPE_CHECKING:
        data: _TodoData

    @override
    def __str__(self) -> str:
        return f"[todo:{self.data!r}]"


class _VoteData(TypedDict):
    topic: str
    options: List[str]


@dataclass
class Vote(MessageSegment):
    if TYPE_CHECKING:
        data: _VoteData

    @override
    def __str__(self) -> str:
        return f"[vote:{self.data!r}]"


class _HongbaoData(TypedDict):
    text: str


@dataclass
class Hongbao(MessageSegment):
    if TYPE_CHECKING:
        data: _HongbaoData

    @override
    def __str__(self) -> str:
        return f"[hongbao:{self.data['text']!r}]"


class _CalendarData(TypedDict):
    summary: str
    start_time: str
    end_time: str


@dataclass
class ShareCalendarEvent(MessageSegment):
    if TYPE_CHECKING:
        data: _CalendarData

    @override
    def __str__(self) -> str:
        return f"[share_calendar_event:{self.data!r}]"


class Calendar:
    if TYPE_CHECKING:
        data: _CalendarData

    @override
    def __str__(self) -> str:
        return f"[calendar:{self.data!r}]"


class GeneralCalendar:
    if TYPE_CHECKING:
        data: _CalendarData

    @override
    def __str__(self) -> str:
        return f"[general_calendar:{self.data!r}]"


class Message(BaseMessage[MessageSegment]):
    """
    飞书 协议 Message 适配。
    """

    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @override
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @override
    def __radd__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super().__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        yield Text("text", {"text": msg})

    def serialize(self) -> Tuple[str, str]:
        combined = {"zh_cn": {"title": "", "content": [[]]}}
        if len(self) >= 2:
            for seg in self:
                if seg.type != "post":
                    seg.to_post()
                    combined["zh_cn"]["content"][-1].append(
                        {"tag": seg.type, **seg.data}
                    )
                else:
                    zh_cn_data = seg.data.pop("zh_cn", None)
                    if zh_cn_data:
                        combined["zh_cn"]["title"] = zh_cn_data["title"]
                        combined["zh_cn"]["content"] = [
                            *combined["zh_cn"]["content"],
                            *zh_cn_data["content"],
                        ]
                    combined.update(seg.data)

            return "post", json.dumps(combined, ensure_ascii=False)
        else:
            return self[0].type, json.dumps(self[0].data, ensure_ascii=False)

    @staticmethod
    def deserialize(
        content: str, mentions: Optional[List[Mention]], message_type: str
    ) -> "Message":
        msg = Message()
        parsed_content = json.loads(content)
        at_key_to_id = {
            # wipeout @ at the start of key
            mention.key[1:]: mention.id.open_id
            for mention in (mentions or [])
        }

        if message_type == "text":
            text: str = parsed_content["text"]
            text_begin = 0

            for embed in re.finditer(
                r"(?P<type>(?:@))(?P<key>\w+)",
                text,
            ):
                matched: str = text[text_begin : embed.pos + embed.start()]
                if matched:
                    msg.extend(Message(Text("text", {"text": matched})))

                text_begin = embed.pos + embed.end()
                if embed.group("type") == "@":
                    msg.extend(
                        Message(
                            At(
                                "at",
                                {"user_id": at_key_to_id.get(embed.group("key"), "")},
                            )
                        )
                    )

            matched = text[text_begin:]
            if matched:
                msg.append(Text("text", {"text": text[text_begin:]}))

        elif message_type == "post":
            msg.append(Post("post", parsed_content))

        else:
            seg_builder = getattr(MessageSegment, message_type, None)
            if seg_builder:
                msg.append(seg_builder(**parsed_content))
            else:
                msg.append(MessageSegment(message_type, parsed_content))

        return msg

    @override
    def extract_plain_text(self) -> str:
        text_list: List[str] = []
        for seg in self:
            if seg.is_text():
                text_list.append(str(seg))

            elif seg.type == "post":
                text_list.append(seg.data["title"])
                for node in itertools.chain.from_iterable(seg.data["content"]):
                    if node["tag"] == "text":
                        text_list.append(node["text"])

        return "".join(text_list)



from typing import Optional

from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable


class FeishuAdapterException(AdapterException):
    def __init__(self):
        super().__init__("feishu")


class ActionFailed(BaseActionFailed, FeishuAdapterException):
    """
    :说明:

      API 请求返回错误信息。

    :参数:

      * ``retcode: Optional[int]``: 错误码
    """

    def __init__(self, **kwargs):
        super().__init__()
        self.info = kwargs

    def __repr__(self):
        return (
            "<ActionFailed " + ", ".join(f"{k}={v}" for k, v in self.info.items()) + ">"
        )

    def __str__(self):
        return self.__repr__()


class NetworkError(BaseNetworkError, FeishuAdapterException):
    """
    :说明:

      网络错误。

    :参数:

      * ``retcode: Optional[int]``: 错误码
    """

    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg = msg

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, FeishuAdapterException):
    pass



from typing import List

from pydantic import HttpUrl, BaseModel

from .common import Reply


class BaseResponse(BaseModel):
    code: int
    msg: str


class TenantAccessTokenResponse(BaseResponse):
    tenant_access_token: str
    expire: int


class BotInfo(BaseModel):
    activate_status: int
    app_name: str
    avatar_url: HttpUrl
    ip_white_list: List[str]
    open_id: str


class BotInfoResponse(BaseResponse):
    bot: BotInfo


class ReplyResponseItems(BaseModel):
    items: List[Reply]


class ReplyResponse(BaseResponse):
    data: ReplyResponseItems


__all__ = [
    "BaseResponse",
    "TenantAccessTokenResponse",
    "BotInfo",
    "BotInfoResponse",
    "ReplyResponseItems",
    "ReplyResponse",
]



import json
import asyncio
import inspect
from typing_extensions import override
from typing import Any, Dict, List, Type, Union, Callable, Optional, cast

from pygtrie import StringTrie
from pydantic import parse_obj_as
from nonebot.utils import escape_tag
from nonebot.drivers import (
    URL,
    Driver,
    Request,
    Response,
    ASGIMixin,
    HTTPClientMixin,
    HTTPServerSetup,
)

from nonebot.adapters import Adapter as BaseAdapter

from . import event
from .bot import Bot
from .event import Event
from .config import Config, BotConfig
from .utils import AESCipher, log, cache
from .message import Message, MessageSegment
from .models import BotInfoResponse, TenantAccessTokenResponse
from .exception import NetworkError, ApiNotAvailable, FeishuAdapterException


class Adapter(BaseAdapter):
    # init all event models
    event_models: StringTrie = StringTrie(separator=".")
    """所有事件模型索引"""

    for model_name in dir(event):
        model = getattr(event, model_name)
        if not inspect.isclass(model) or not issubclass(model, Event):
            continue
        event_models["." + model.__event__] = model

    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)
        """飞书适配器配置"""
        self.feishu_config: Config = Config(**self.config.dict())
        self.bot_apps: Dict[str, BotConfig] = {}
        self.setup()

    @classmethod
    @override
    def get_name(cls) -> str:
        """适配器名称: `Feishu`"""
        return "Feishu"

    async def startup(self):
        for bot_config in self.feishu_config.feishu_bots:
            self.bot_apps[bot_config.app_id] = bot_config
            result = await self.get_bot_info(bot_config)
            if result.code != 0:
                log(
                    "ERROR",
                    "<r><bg #f8bbd0>Failed to get bot info.</bg #f8bbd0></r> "
                    f"Skipped Bot {bot_config.app_id} registration",
                )
                continue

            if self.bots.get(bot_config.app_id):
                continue

            bot = Bot(
                self,
                bot_config.app_id,
                bot_config=bot_config,
                bot_info=result.bot,
            )
            self.bot_connect(bot)
            log(
                "INFO",
                f"<y>Bot {escape_tag(bot_config.app_id)}</y> connected",
            )

            setup = HTTPServerSetup(
                URL(f"/feishu/{bot.self_id}"),
                "POST",
                self.get_name(),
                self._handle_http,
            )
            self.setup_http_server(setup)

    def setup(self) -> None:
        if not isinstance(self.driver, ASGIMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} "
                "doesn't support reverse connections!"
                f"{self.get_name()} Adapter needs a ASGI Driver to work."
            )

        if not isinstance(self.driver, HTTPClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} "
                "doesn't support http client requests!"
                f"{self.get_name()} Adapter needs a HTTPClient Driver to work."
            )

        self.driver.on_startup(self.startup)

    def get_api_url(self, bot_config: BotConfig, path: str):
        api_base = (
            self.feishu_config.feishu_lark_api_base
            if bot_config.is_lark
            else self.feishu_config.feishu_api_base
        )

        return api_base + path

    async def get_bot_info(self, bot_config: BotConfig):
        token = await self.get_tenant_access_token(bot_config)

        response = await self.send_request(
            Request(
                "GET",
                self.get_api_url(bot_config, "bot/v3/info"),
                headers={
                    "Authorization": f"Bearer {token}",
                },
            ),
        )
        result = parse_obj_as(BotInfoResponse, response)

        return result

    async def get_tenant_access_token(self, bot_config: BotConfig):
        key = f"feishu_tenant_access_token_{bot_config.app_id}"

        if token := await cache.get(key):
            return cast(str, token)

        response = await self.send_request(
            Request(
                "POST",
                self.get_api_url(bot_config, "auth/v3/tenant_access_token/internal/"),
                json={
                    "app_id": bot_config.app_id,
                    "app_secret": bot_config.app_secret,
                },
            ),
        )
        result = parse_obj_as(TenantAccessTokenResponse, response)

        expire = result.expire
        if expire > 30 * 60:
            expire -= 30 * 60

        await cache.set(key, result.tenant_access_token, expire)

        return result.tenant_access_token

    async def send_request(self, request: Request, **data: Any):
        timeout: float = data.get("_timeout", self.config.api_timeout)
        request.timeout = timeout

        if not isinstance(self.driver, HTTPClientMixin):
            raise ApiNotAvailable

        try:
            response = await self.driver.request(request)

            if 200 <= response.status_code < 300:
                if not response.content:
                    raise ValueError("Empty response")

                if response.headers["Content-Type"].find("application/json") != -1:
                    result = json.loads(response.content)
                    return result
                else:
                    return response.content

            raise NetworkError(
                f"HTTP request received unexpected "
                f"status code: {response.status_code}"
            )

        except FeishuAdapterException:
            raise

        except Exception as e:
            raise NetworkError("HTTP request failed") from e

    @override
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        token = await self.get_tenant_access_token(bot.bot_config)
        headers = {**data.get("headers", {}), "Authorization": f"Bearer {token}"}

        request = Request(
            data["method"],
            self.get_api_url(bot.bot_config, api),
            cookies=data.get("cookies"),
            content=data.get("content"),
            data=data.get("data"),
            files=data.get("files"),
            json=data.get("json") or data.get("body"),
            params=data.get("params") or data.get("query"),
            headers=headers,
        )

        return await self.send_request(request, **data)

    async def _handle_http(self, request: Request) -> Response:
        bot_config = self.bot_apps.get(request.url.parts[-1])
        if bot_config is None:
            raise RuntimeError("Corresponding bot config not found")

        if (data := request.content) is not None:
            if bot_config.encrypt_key is not None:
                if (encrypted := json.loads(data).get("encrypt")) is not None:
                    decrypted = AESCipher(bot_config.encrypt_key).decrypt_string(
                        encrypted
                    )
                    data = json.loads(decrypted)
            else:
                data = json.loads(data)

        if not isinstance(data, dict):
            return Response(500, content="Received non-JSON data, cannot cast to dict")

        if challenge := data.get("challenge"):
            return Response(200, content=json.dumps({"challenge": challenge}).encode())

        if not data.get("schema"):
            return Response(
                400,
                content=("Missing `schema` in POST body, only accept event V2"),
            )

        if headers := data.get("header"):
            token: str = headers.get("token")
        else:
            log("WARNING", "Missing `header` in POST body")
            return Response(
                400,
                content="Missing `header` in POST body",
            )

        if not token:
            log("WARNING", "Missing `verification token` in POST body")
            return Response(
                400,
                content="Missing `verification token` in POST body",
            )
        else:
            if token != bot_config.verification_token:
                log("WARNING", "Verification token check failed")
                return Response(
                    403,
                    content="Verification token check failed",
                )

        if data is not None:
            if not (bot := self.bots.get(bot_config.app_id)):
                raise RuntimeError("Corresponding Bot instance not found")

            if event := self.json_to_event(data):
                asyncio.create_task(cast(Bot, bot).handle_event(event))

        return Response(200)

    @classmethod
    def json_to_event(cls, json_data: Any) -> Optional[Event]:
        """将 json 数据转换为 Event 对象。
        参数:
            json_data: json 数据
            self_id: 当前 Event 对应的 Bot
        返回:
            Event 对象，如果解析失败则返回 None
        """
        if not isinstance(json_data, dict):
            return

        if json_data.get("type") == "url_verification":
            return

        try:
            header = json_data["header"]
            event_type = header.get("event_type", "")
            if json_data.get("event", {}).get("message"):
                event_type += f".{json_data['event']['message']['chat_type']}"

            models = cls.get_event_model(event_type)
            for model in models:
                try:
                    event = model.parse_obj(json_data)
                    break
                except Exception as e:
                    log("DEBUG", "Event Parser Error", e)
            else:
                event = Event.parse_obj(json_data)

            return event

        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to parse event. "
                f"Raw: {escape_tag(str(json_data))}</bg #f8bbd0></r>",
                e,
            )

    @classmethod
    def add_custom_model(cls, model: Type[Event]) -> None:
        """插入或覆盖一个自定义的 Event 类型。
        需提供 `__event__` 属性，进行事件模型索引，
        格式为 `{post_type}[.{sub_type}]`，如: `message.private`。
        参数:
            model: 自定义的 Event 类型
        """
        if not hasattr(model, "__event__"):
            raise ValueError("Event model's `__event__` attribute must be set")
        cls.event_models["." + model.__event__] = model

    @classmethod
    def get_event_model(cls, event_name: str) -> List[Type[Event]]:
        """根据事件名获取对应 `Event Model` 及 `FallBack Event Model` 列表，
        不包括基类 `Event`。
        """
        return [model.value for model in cls.event_models.prefixes("." + event_name)][
            ::-1
        ]

    @classmethod
    def custom_send(
        cls,
        send_func: Callable[[Bot, Event, Union[str, Message, MessageSegment]], Any],
    ):
        """自定义 Bot 的回复函数。"""
        setattr(Bot, "send_handler", send_func)



from typing import List, Optional

from pydantic import Field, HttpUrl, BaseModel


class BotConfig(BaseModel):
    """
    飞书适配器机器人配置类

    :配置项:

      - ``app_id``: 飞书开放平台后台“凭证与基础信息”处给出的 App ID
      - ``app_secret``: 飞书开放平台后台“凭证与基础信息”处给出的 App Secret
      - ``encrypt_key``: 飞书开放平台后台“事件订阅”处设置的 Encrypt Key
      - ``verification_token``: 飞书开放平台后台“事件订阅”处设置的 Verification Token
      - ``is_lark``: 是否使用 Lark（飞书海外版），默认为 false

    """

    app_id: str
    app_secret: str
    encrypt_key: Optional[str] = None
    verification_token: str
    is_lark: bool = False


class Config(BaseModel):
    """
    飞书适配器全局配置类

    :配置项:

      - ``feishu_api_base``: 飞书国内版开放平台 API Endpoint
      - ``feishu_lark_api_base``: 飞书海外版（lark）开放平台 API Endpoint
      - ``feishu_bots``: 飞书适配器 Bot 配置列表，具体配置项参阅 BotConfig 类

    """

    feishu_api_base: HttpUrl = Field("https://open.feishu.cn/open-apis/")
    feishu_lark_api_base: HttpUrl = Field("https://open.larksuite.com/open-apis/")
    feishu_bots: List[BotConfig] = Field(default_factory=list)

    class Config:
        extra = "ignore"



from typing import List, Literal, Optional

from pydantic import BaseModel


class EventHeader(BaseModel):
    event_id: str
    event_type: str
    create_time: str
    token: str
    app_id: str
    tenant_key: str
    resource_id: Optional[str]
    user_list: Optional[List[dict]]

    class Config:
        extra = "allow"


class UserId(BaseModel):
    union_id: str
    user_id: str
    open_id: str


class MeetingUser(BaseModel):
    id: UserId
    user_role: Optional[int]
    user_type: Optional[int]


class Meeting(BaseModel):
    id: str
    topic: str
    meeting_no: str
    start_time: Optional[str]
    end_time: Optional[str]
    host_user: Optional[MeetingUser]
    owner: MeetingUser


class VCMeetingRecordingReadyEventDetail(BaseModel):
    meeting: Meeting
    url: str
    duration: str


class VCMeetingRecordingEndedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class VCMeetingRecordingStartedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class VCMeetingLeftEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser
    leave_reason: int


class VCMeetingJoinedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class VCMeetingEndedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class Sender(BaseModel):
    sender_id: UserId
    sender_type: str
    tenant_key: str


class ReplySender(BaseModel):
    id: str
    id_type: str
    sender_type: str
    tenant_key: str


class Mention(BaseModel):
    key: str
    id: UserId
    name: str
    tenant_key: str


class ReplyMention(BaseModel):
    id: str
    id_type: str
    key: str
    name: str
    tenant_key: str


class MessageBody(BaseModel):
    content: str


class Reply(BaseModel):
    message_id: str
    root_id: Optional[str]
    parent_id: Optional[str]
    msg_type: str
    create_time: str
    update_time: str
    deleted: bool
    updated: bool
    chat_id: str
    sender: ReplySender
    body: MessageBody
    mentions: List[ReplyMention]
    upper_message_id: Optional[str]

    class Config:
        extra = "allow"


class EventMessage(BaseModel):
    message_id: str
    root_id: Optional[str]
    parent_id: Optional[str]
    create_time: str
    chat_id: str
    chat_type: str
    message_type: str
    content: str
    mentions: Optional[List[Mention]]

    class Config:
        extra = "allow"


class GroupEventMessage(EventMessage):
    chat_type: Literal["group"]


class PrivateEventMessage(EventMessage):
    chat_type: Literal["p2p"]


class MessageEventDetail(BaseModel):
    sender: Sender
    message: EventMessage


class GroupMessageEventDetail(MessageEventDetail):
    message: GroupEventMessage


class PrivateMessageEventDetail(MessageEventDetail):
    message: PrivateEventMessage


class VCMeetingShareStartedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class VCMeetingShareEndedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class AttendanceUserFlowCreatedEventDetail(BaseModel):
    bssid: str
    check_time: str
    comment: str
    employee_id: str
    employee_no: str
    is_field: bool
    is_wifi: bool
    latitude: float
    location_name: str
    longitude: float
    photo_urls: Optional[List[str]]
    record_id: str
    ssid: str
    type: int


class AttendanceUserTaskStatusDiff(BaseModel):
    before_status: str
    before_supplement: str
    current_status: str
    current_supplement: str
    index: int
    work_type: str


class AttendanceUserTaskUpdatedEventDetail(BaseModel):
    date: int
    employee_id: str
    employee_no: str
    group_id: str
    shift_id: str
    status_changes: List[AttendanceUserTaskStatusDiff]
    task_id: str
    time_zone: str


class VCMeetingStartedEventDetail(BaseModel):
    meeting: Meeting
    operator: MeetingUser


class MeetingRoomStatusChangedEventDetail(BaseModel):
    room_id: str
    room_name: str


class MeetingRoomDeletedEventDetail(BaseModel):
    room_id: str
    room_name: str


class MeetingRoomUpdatedEventDetail(BaseModel):
    room_id: str
    room_name: str


class MeetingRoomCreatedEventDetail(BaseModel):
    room_id: str
    room_name: str


class DriveFileEditedEventDetail(BaseModel):
    file_token: str
    file_type: str
    operator_id_list: List[UserId]
    subscriber_id_list: List[UserId]


class DriveFileDeletedEventDetail(BaseModel):
    file_token: str
    file_type: str
    operator_id: UserId


class DriveFileTrashedEventDetail(BaseModel):
    file_token: str
    file_type: str
    operator_id: UserId


class DriveFilePermissionMemberRemovedEventDetail(BaseModel):
    chat_list: List[str]
    file_token: str
    file_type: str
    operator_id: UserId
    user_list: List[UserId]


class DriveFilePermissionMemberAddedEventDetail(BaseModel):
    chat_list: List[str]
    file_token: str
    file_type: str
    operator_id: UserId
    user_list: List[UserId]


class DriveFileTitleUpdatedEventDetail(BaseModel):
    file_token: str
    file_type: str
    operator_id: UserId


class DriveFileReadEventDetail(BaseModel):
    file_token: str
    file_type: str
    operator_id_list: List[UserId]


class CalendarEventChangedEventDetail(BaseModel):
    calendar_id: str


class CalendarAclScope(BaseModel):
    type: str
    user_id: str


class CalendarAclDeletedEventDetail(BaseModel):
    acl_id: str
    role: str
    scope: CalendarAclScope


class CalendarAclCreatedEventDetail(BaseModel):
    acl_id: str
    role: str
    scope: CalendarAclScope


class ContactDepartmentStatus(BaseModel):
    is_deleted: bool


class ContactDepartment(BaseModel):
    name: str
    parent_department_id: str
    department_id: str
    open_department_id: str
    leader_user_id: str
    chat_id: str
    order: int
    status: ContactDepartmentStatus


class ContactDepartmentCreatedEventDetail(BaseModel):
    object: ContactDepartment


class OldContactDepartment(BaseModel):
    status: ContactDepartmentStatus
    open_department_id: str


class ContactDepartmentDeletedEventDetail(BaseModel):
    object: ContactDepartment
    old_object: OldContactDepartment


class ContactDepartmentUpdatedEventDetail(BaseModel):
    object: ContactDepartment
    old_object: ContactDepartment


class AvatarInfo(BaseModel):
    avatar_72: str
    avatar_240: str
    avatar_640: str
    avatar_origin: str


class UserStatus(BaseModel):
    is_frozen: bool
    is_resigned: bool
    is_activated: bool


class UserOrder(BaseModel):
    department_id: str
    user_order: int
    department_order: int


class UserCustomAttrValue(BaseModel):
    text: str
    url: str
    pc_url: str


class UserCustomAttr(BaseModel):
    type: str
    id: str
    value: UserCustomAttrValue


class ContactUser(BaseModel):
    open_id: str
    user_id: str
    name: str
    en_name: str
    email: str
    mobile: str
    gender: int
    avatar: AvatarInfo
    status: UserStatus
    department_ids: Optional[List[str]]
    leader_user_id: str
    city: str
    country: str
    work_station: str
    join_time: int
    employee_no: str
    employee_type: int
    orders: Optional[List[UserOrder]]
    custom_attrs: List[UserCustomAttr]


class OldContactUser(BaseModel):
    department_ids: List[str]
    open_id: str


class ContactUserCreatedEventDetail(BaseModel):
    object: ContactUser


class ContactUserDeletedEventDetail(BaseModel):
    object: ContactUser
    old_object: OldContactUser


class ContactUserUpdatedEventDetail(BaseModel):
    object: ContactUser
    old_object: ContactUser


class ChatMemberUser(BaseModel):
    name: str
    tenant_key: str
    user_id: UserId


class GroupMemberUserDeletedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str
    users: List[ChatMemberUser]


class GroupMemberUserWithdrawnEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str
    users: List[ChatMemberUser]


class GroupMemberUserAddedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str
    users: List[ChatMemberUser]


class GroupMemberBotDeletedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str


class GroupMemberBotAddedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str


class I18nNames(BaseModel):
    zh_cn: str
    en_us: str
    ja_jp: str


class Emoji(BaseModel):
    emoji_type: str


class ChatChange(BaseModel):
    avatar: str
    name: str
    description: str
    i18n_names: I18nNames
    add_member_permission: str
    share_card_permission: str
    at_all_permission: str
    edit_permission: str
    membership_approval: str
    join_message_visibility: str
    leave_message_visibility: str
    moderation_permission: str
    owner_id: UserId


class EventModerator(BaseModel):
    tenant_key: str
    user_id: UserId


class ModeratorList(BaseModel):
    added_member_list: EventModerator
    removed_member_list: EventModerator


class GroupConfigUpdatedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str
    after_change: ChatChange
    before_change: ChatChange
    moderator_list: ModeratorList


class MessageReactionCreatedEventDetail(BaseModel):
    message_id: str
    reaction_type: Emoji
    operator_type: str
    user_id: UserId
    action_time: str


class MessageReactionDeletedEventDetail(BaseModel):
    message_id: str
    reaction_type: Emoji
    operator_type: str
    user_id: UserId
    action_time: str


class GroupDisbandedEventDetail(BaseModel):
    chat_id: str
    operator_id: UserId
    external: bool
    operator_tenant_key: str


class MessageReader(BaseModel):
    reader_id: UserId
    read_time: str
    tenant_key: str


class MessageReadEventDetail(BaseModel):
    reader: MessageReader
    message_id_list: List[str]


__all__ = [
    "EventHeader",
    "UserId",
    "MeetingUser",
    "Meeting",
    "VCMeetingRecordingReadyEventDetail",
    "VCMeetingRecordingEndedEventDetail",
    "VCMeetingRecordingStartedEventDetail",
    "VCMeetingLeftEventDetail",
    "VCMeetingJoinedEventDetail",
    "VCMeetingEndedEventDetail",
    "Sender",
    "ReplySender",
    "Mention",
    "ReplyMention",
    "MessageBody",
    "Reply",
    "EventMessage",
    "GroupEventMessage",
    "PrivateEventMessage",
    "MessageEventDetail",
    "GroupMessageEventDetail",
    "PrivateMessageEventDetail",
    "VCMeetingShareStartedEventDetail",
    "VCMeetingShareEndedEventDetail",
    "AttendanceUserFlowCreatedEventDetail",
    "AttendanceUserTaskStatusDiff",
    "AttendanceUserTaskUpdatedEventDetail",
    "VCMeetingStartedEventDetail",
    "MeetingRoomStatusChangedEventDetail",
    "MeetingRoomDeletedEventDetail",
    "MeetingRoomUpdatedEventDetail",
    "MeetingRoomCreatedEventDetail",
    "DriveFileEditedEventDetail",
    "DriveFileDeletedEventDetail",
    "DriveFileTrashedEventDetail",
    "DriveFilePermissionMemberRemovedEventDetail",
    "DriveFilePermissionMemberAddedEventDetail",
    "DriveFileTitleUpdatedEventDetail",
    "DriveFileReadEventDetail",
    "CalendarEventChangedEventDetail",
    "CalendarAclScope",
    "CalendarAclDeletedEventDetail",
    "CalendarAclCreatedEventDetail",
    "ContactDepartmentStatus",
    "ContactDepartment",
    "ContactDepartmentCreatedEventDetail",
    "OldContactDepartment",
    "ContactDepartmentDeletedEventDetail",
    "ContactDepartmentUpdatedEventDetail",
    "AvatarInfo",
    "UserStatus",
    "UserOrder",
    "UserCustomAttrValue",
    "UserCustomAttr",
    "ContactUser",
    "OldContactUser",
    "ContactUserCreatedEventDetail",
    "ContactUserDeletedEventDetail",
    "ContactUserUpdatedEventDetail",
    "ChatMemberUser",
    "GroupMemberUserDeletedEventDetail",
    "GroupMemberUserWithdrawnEventDetail",
    "GroupMemberUserAddedEventDetail",
    "GroupMemberBotDeletedEventDetail",
    "GroupMemberBotAddedEventDetail",
    "I18nNames",
    "Emoji",
    "ChatChange",
    "EventModerator",
    "ModeratorList",
    "GroupConfigUpdatedEventDetail",
    "MessageReactionCreatedEventDetail",
    "MessageReactionDeletedEventDetail",
    "GroupDisbandedEventDetail",
    "MessageReader",
    "MessageReadEventDetail",
]



from copy import deepcopy
from datetime import datetime
from typing_extensions import override
from typing import TYPE_CHECKING, Any, Dict, Literal, Optional

from pydantic import Field
from nonebot.utils import escape_tag

from nonebot.adapters import Event as BaseEvent

from .message import Message
from .models import (
    Reply,
    UserId,
    EventHeader,
    MessageEventDetail,
    MessageReadEventDetail,
    GroupMessageEventDetail,
    DriveFileReadEventDetail,
    VCMeetingLeftEventDetail,
    GroupDisbandedEventDetail,
    PrivateMessageEventDetail,
    VCMeetingEndedEventDetail,
    DriveFileEditedEventDetail,
    VCMeetingJoinedEventDetail,
    DriveFileDeletedEventDetail,
    DriveFileTrashedEventDetail,
    VCMeetingStartedEventDetail,
    CalendarAclCreatedEventDetail,
    CalendarAclDeletedEventDetail,
    ContactUserCreatedEventDetail,
    ContactUserDeletedEventDetail,
    ContactUserUpdatedEventDetail,
    GroupConfigUpdatedEventDetail,
    MeetingRoomCreatedEventDetail,
    MeetingRoomDeletedEventDetail,
    MeetingRoomUpdatedEventDetail,
    GroupMemberBotAddedEventDetail,
    VCMeetingShareEndedEventDetail,
    CalendarEventChangedEventDetail,
    GroupMemberUserAddedEventDetail,
    DriveFileTitleUpdatedEventDetail,
    GroupMemberBotDeletedEventDetail,
    VCMeetingShareStartedEventDetail,
    GroupMemberUserDeletedEventDetail,
    MessageReactionCreatedEventDetail,
    MessageReactionDeletedEventDetail,
    VCMeetingRecordingEndedEventDetail,
    VCMeetingRecordingReadyEventDetail,
    ContactDepartmentCreatedEventDetail,
    ContactDepartmentDeletedEventDetail,
    ContactDepartmentUpdatedEventDetail,
    GroupMemberUserWithdrawnEventDetail,
    MeetingRoomStatusChangedEventDetail,
    AttendanceUserFlowCreatedEventDetail,
    AttendanceUserTaskUpdatedEventDetail,
    VCMeetingRecordingStartedEventDetail,
    DriveFilePermissionMemberAddedEventDetail,
    DriveFilePermissionMemberRemovedEventDetail,
)


class Event(BaseEvent):
    """
    飞书协议事件。各事件字段参考 `飞书文档`_

    .. _飞书文档:
        https://open.feishu.cn/document/ukTMukTMukTM/uYDNxYjL2QTM24iN0EjN/event-list
    """

    __event__ = ""
    schema_: str = Field("", alias="schema")
    header: EventHeader
    event: Any

    @override
    def get_type(self) -> str:
        return self.header.event_type

    @override
    def get_event_name(self) -> str:
        return self.header.event_type

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(self.dict()))

    @override
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @override
    def get_plaintext(self) -> str:
        raise ValueError("Event has no plaintext!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no user_id!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no session_id!")

    @override
    def is_tome(self) -> bool:
        return False

    @property
    def time(self) -> datetime:
        return datetime.utcfromtimestamp(int(self.header.create_time) / 1000)


class MessageEvent(Event):
    __event__ = "im.message.receive_v1"
    event: MessageEventDetail

    to_me: bool = False
    """
    :说明: 消息是否与机器人有关

    :类型: ``bool``
    """
    reply: Optional[Reply]

    if TYPE_CHECKING:
        _message: Message
        original_message: Message

    @override
    def get_type(self) -> Literal["message"]:
        return "message"

    @override
    def get_event_name(self) -> str:
        return f"{self.get_type()}.{self.event.message.chat_type}"

    @override
    def get_event_description(self) -> str:
        return (
            f"{self.event.message.message_id} from {self.get_user_id()}"
            f"@[{self.event.message.chat_type}:{self.event.message.chat_id}]"
            f" {escape_tag(str(self.get_message()))}"
        )

    @override
    def get_message(self) -> Message:
        if not hasattr(self, "_message"):
            deserialized = Message.deserialize(
                self.event.message.content,
                self.event.message.mentions,
                self.event.message.message_type,
            )
            setattr(
                self,
                "_message",
                deserialized,
            )
            setattr(
                self,
                "original_message",
                deepcopy(deserialized),
            )

        return getattr(self, "_message")

    @property
    def message_id(self) -> str:
        return self.event.message.message_id

    @override
    def get_plaintext(self) -> str:
        return str(self.get_message())

    @override
    def get_user_id(self) -> str:
        return self.event.sender.sender_id.open_id

    def get_all_user_id(self) -> UserId:
        return self.event.sender.sender_id

    @override
    def get_session_id(self) -> str:
        return (
            f"{self.event.message.chat_type}"
            f"_{self.event.message.chat_id}"
            f"_{self.get_user_id()}"
        )

    @override
    def is_tome(self) -> bool:
        return self.to_me


class GroupMessageEvent(MessageEvent):
    __event__ = "im.message.receive_v1.group"
    event: GroupMessageEventDetail


class PrivateMessageEvent(MessageEvent):
    __event__ = "im.message.receive_v1.p2p"
    event: PrivateMessageEventDetail


class NoticeEvent(Event):
    event: Dict[str, Any]

    @override
    def get_type(self) -> Literal["notice"]:
        return "notice"

    @override
    def get_event_name(self) -> str:
        return self.header.event_type

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(self.dict()))

    @override
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @override
    def get_plaintext(self) -> str:
        raise ValueError("Event has no plaintext!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no user_id!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no session_id!")


class MessageReadEvent(NoticeEvent):
    __event__ = "im.message.message_read_v1"
    event: MessageReadEventDetail


class GroupDisbandedEvent(NoticeEvent):
    __event__ = "im.chat.disbanded_v1"
    event: GroupDisbandedEventDetail


class MessageReactionDeletedEvent(NoticeEvent):
    __event__ = "im.message.reaction.deleted_v1"
    event: MessageReactionDeletedEventDetail


class MessageReactionCreatedEvent(NoticeEvent):
    __event__ = "im.message.reaction.created_v1"
    event: MessageReactionCreatedEventDetail


class GroupConfigUpdatedEvent(NoticeEvent):
    __event__ = "im.chat.updated_v1"
    event: GroupConfigUpdatedEventDetail


class GroupMemberBotAddedEvent(NoticeEvent):
    __event__ = "im.chat.member.bot.added_v1"
    event: GroupMemberBotAddedEventDetail


class GroupMemberBotDeletedEvent(NoticeEvent):
    __event__ = "im.chat.member.bot.deleted_v1"
    event: GroupMemberBotDeletedEventDetail


class GroupMemberUserAddedEvent(NoticeEvent):
    __event__ = "im.chat.member.user.added_v1"
    event: GroupMemberUserAddedEventDetail


class GroupMemberUserWithdrawnEvent(NoticeEvent):
    __event__ = "im.chat.member.user.withdrawn_v1"
    event: GroupMemberUserWithdrawnEventDetail


class GroupMemberUserDeletedEvent(NoticeEvent):
    __event__ = "im.chat.member.user.deleted_v1"
    event: GroupMemberUserDeletedEventDetail


class ContactUserUpdatedEvent(NoticeEvent):
    __event__ = "contact.user.updated_v3"
    event: ContactUserUpdatedEventDetail


class ContactUserDeletedEvent(NoticeEvent):
    __event__ = "contact.user.deleted_v3"
    event: ContactUserDeletedEventDetail


class ContactUserCreatedEvent(NoticeEvent):
    __event__ = "contact.user.created_v3"
    event: ContactUserCreatedEventDetail


class ContactDepartmentUpdatedEvent(NoticeEvent):
    __event__ = "contact.department.updated_v3"
    event: ContactDepartmentUpdatedEventDetail


class ContactDepartmentDeletedEvent(NoticeEvent):
    __event__ = "contact.department.deleted_v3"
    event: ContactDepartmentDeletedEventDetail


class ContactDepartmentCreatedEvent(NoticeEvent):
    __event__ = "contact.department.created_v3"
    event: ContactDepartmentCreatedEventDetail


class CalendarAclCreatedEvent(NoticeEvent):
    __event__ = "calendar.calendar.acl.created_v4"
    event: CalendarAclCreatedEventDetail


class CalendarAclDeletedEvent(NoticeEvent):
    __event__ = "calendar.calendar.acl.deleted_v4"
    event: CalendarAclDeletedEventDetail


class CalendarChangedEvent(NoticeEvent):
    __event__ = "calendar.calendar.changed_v4"
    event: dict


class CalendarEventChangedEvent(NoticeEvent):
    __event__ = "calendar.calendar.event.changed_v4"
    event: CalendarEventChangedEventDetail


class DriveFileReadEvent(NoticeEvent):
    __event__ = "drive.file.read_v1"
    event: DriveFileReadEventDetail


class DriveFileTitleUpdatedEvent(NoticeEvent):
    __event__ = "drive.file.title_updated_v1"
    event: DriveFileTitleUpdatedEventDetail


class DriveFilePermissionMemberAddedEvent(NoticeEvent):
    __event__ = "drive.file.permission_member_added_v1"
    event: DriveFilePermissionMemberAddedEventDetail


class DriveFilePermissionMemberRemovedEvent(NoticeEvent):
    __event__ = "drive.file.permission_member_removed_v1"
    event: DriveFilePermissionMemberRemovedEventDetail


class DriveFileTrashedEvent(NoticeEvent):
    __event__ = "drive.file.trashed_v1"
    event: DriveFileTrashedEventDetail


class DriveFileDeletedEvent(NoticeEvent):
    __event__ = "drive.file.deleted_v1"
    event: DriveFileDeletedEventDetail


class DriveFileEditedEvent(NoticeEvent):
    __event__ = "drive.file.edit_v1"
    event: DriveFileEditedEventDetail


class MeetingRoomCreatedEvent(NoticeEvent):
    __event__ = "meeting_room.meeting_room.created_v1"
    event: MeetingRoomCreatedEventDetail


class MeetingRoomUpdatedEvent(NoticeEvent):
    __event__ = "meeting_room.meeting_room.updated_v1"
    event: MeetingRoomUpdatedEventDetail


class MeetingRoomDeletedEvent(NoticeEvent):
    __event__ = "meeting_room.meeting_room.deleted_v1"
    event: MeetingRoomDeletedEventDetail


class MeetingRoomStatusChangedEvent(NoticeEvent):
    __event__ = "meeting_room.meeting_room.status_changed_v1"
    event: MeetingRoomStatusChangedEventDetail


class VCMeetingStartedEvent(NoticeEvent):
    __event__ = "vc.meeting.meeting_started_v1"
    event: VCMeetingStartedEventDetail


class VCMeetingEndedEvent(NoticeEvent):
    __event__ = "vc.meeting.meeting_ended_v1"
    event: VCMeetingEndedEventDetail


class VCMeetingJoinedEvent(NoticeEvent):
    __event__ = "vc.meeting.join_meeting_v1"
    event: VCMeetingJoinedEventDetail


class VCMeetingLeftEvent(NoticeEvent):
    __event__ = "vc.meeting.leave_meeting_v1"
    event: VCMeetingLeftEventDetail


class VCMeetingRecordingStartedEvent(NoticeEvent):
    __event__ = "vc.meeting.recording_started_v1"
    event: VCMeetingRecordingStartedEventDetail


class VCMeetingRecordingEndedEvent(NoticeEvent):
    __event__ = "vc.meeting.recording_ended_v1"
    event: VCMeetingRecordingEndedEventDetail


class VCMeetingRecordingReadyEvent(NoticeEvent):
    __event__ = "vc.meeting.recording_ready_v1"
    event: VCMeetingRecordingReadyEventDetail


class VCMeetingShareStartedEvent(NoticeEvent):
    __event__ = "vc.meeting.share_started_v1"
    event: VCMeetingShareStartedEventDetail


class VCMeetingShareEndedEvent(NoticeEvent):
    __event__ = "vc.meeting.share_ended_v1"
    event: VCMeetingShareEndedEventDetail


class AttendanceUserFlowCreatedEvent(NoticeEvent):
    __event__ = "attendance.user_flow.created_v1"
    event: AttendanceUserFlowCreatedEventDetail


class AttendanceUserTaskUpdatedEvent(NoticeEvent):
    __event__ = "attendance.user_task.updated_v1"
    event: AttendanceUserTaskUpdatedEventDetail



import re
from typing_extensions import override
from typing import TYPE_CHECKING, Any, Dict, List, Union, Literal, Callable, Optional

from pydantic import parse_obj_as
from nonebot.message import handle_event

from nonebot.adapters import Bot as BaseBot

from .utils import log
from .config import BotConfig
from .models import BotInfo, ReplyResponse
from .message import At, Message, MessageSegment
from .event import Event, MessageEvent, GroupMessageEvent, PrivateMessageEvent

if TYPE_CHECKING:
    from .adapter import Adapter


async def _check_reply(bot: "Bot", event: "Event"):
    if not isinstance(event, MessageEvent):
        return

    if not event.event.message.parent_id:
        return

    if event.event.message.parent_id != event.event.message.message_id:
        try:
            response = await bot.call_api(
                f"im/v1/messages/{event.event.message.parent_id}",
                method="GET",
            )
            result = parse_obj_as(
                ReplyResponse,
                response,
            )
            for message in result.data.items:
                if (
                    message.sender.id_type == "app_id"
                    and message.sender.id == bot.bot_config.app_id
                ):
                    event.to_me = True
                    event.reply = message
                    return

            if not event.reply and len(result.data.items) >= 1:
                event.reply = result.data.items[0]

        except Exception as e:
            log("ERROR", "Failed to get reply message", e)


def _check_at_me(bot: "Bot", event: "Event"):
    """
    :说明:

      检查消息开头或结尾是否存在 @机器人，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    if not isinstance(event, MessageEvent):
        return

    # ensure message not empty
    if not (message := event.get_message()):
        message.append(MessageSegment.text(""))

    if event.event.message.chat_type == "p2p":
        event.to_me = True
        return

    if (
        isinstance(event, GroupMessageEvent)
        and event.event.message.mentions is not None
        and bot.bot_info.open_id
        in {user.id.open_id for user in event.event.message.mentions}
    ):
        event.to_me = True

    def _is_at_me_seg(segment: MessageSegment) -> bool:
        return (
            segment.type == "at" and segment.data.get("user_id") == bot.bot_info.open_id
        )

    deleted = False
    if _is_at_me_seg(message[0]):
        message.pop(0)
        deleted = True
        event.to_me = True
        if message and message[0].type == "text":
            message[0].data["text"] = message[0].data["text"].lstrip()
            if not message[0].data["text"]:
                del message[0]

    if not deleted:
        # wipe out last empty text segment
        i = -1
        last_msg_seg = message[i]
        if (
            last_msg_seg.type == "text"
            and not last_msg_seg.data["text"].strip()
            and len(message) >= 2
        ):
            i -= 1
            last_msg_seg = message[i]

        if _is_at_me_seg(last_msg_seg):
            del message[i:]
            deleted = True
            event.to_me = True

    if not message:
        message.append(MessageSegment.text(""))


def _check_nickname(bot: "Bot", event: "Event"):
    """
    :说明:

      检查消息开头是否存在昵称，去除并赋值 ``event.to_me``

    :参数:

      * ``bot: Bot``: Bot 对象
      * ``event: Event``: Event 对象
    """
    if not isinstance(event, MessageEvent):
        return

    first_msg_seg = event.get_message()[0]
    if first_msg_seg.type != "text":
        return

    first_text = first_msg_seg.data["text"]

    nicknames = set(filter(lambda n: n, bot.config.nickname))
    if nicknames:
        # check if the user is calling me with my nickname
        nickname_regex = "|".join(nicknames)
        m = re.search(rf"^({nickname_regex})([\s,，]*|$)", first_text, re.IGNORECASE)
        if m:
            nickname = m.group(1)
            log("DEBUG", f"User is calling me {nickname}")
            event.to_me = True
            first_msg_seg.data["text"] = first_text[m.end() :]


async def send(
    bot: "Bot",
    event: Event,
    message: Union[str, Message, MessageSegment],
    at_sender: bool = False,
    **kwargs: Any,
) -> Any:
    """默认回复消息处理函数。"""

    message = message if isinstance(message, Message) else Message(message)

    if isinstance(event, GroupMessageEvent):
        receive_id, receive_id_type = event.event.message.chat_id, "chat_id"
    elif isinstance(event, PrivateMessageEvent):
        receive_id, receive_id_type = event.get_user_id(), "open_id"
    else:
        raise ValueError("Cannot guess `receive_id` and `receive_id_type` to reply!")

    full_message = Message()  # create a new message for prepending
    at_sender = at_sender and bool(event.get_user_id())
    if at_sender and receive_id_type == "chat_id":
        full_message += At("at", {"user_id": event.get_user_id()}) + " "
    full_message += message

    msg_type, content = full_message.serialize()

    return await bot.send_msg(receive_id_type, receive_id, content, msg_type)


class Bot(BaseBot):
    send_handler: Callable[
        ["Bot", Event, Union[str, Message, MessageSegment]], Any
    ] = send

    @override
    def __init__(
        self,
        adapter: "Adapter",
        self_id: str,
        *,
        bot_config: BotConfig,
        bot_info: BotInfo,
    ):
        super().__init__(adapter, self_id)
        self.bot_config: BotConfig = bot_config
        self.bot_info: BotInfo = bot_info

    async def get_msgs(
        self, container_id_type: Literal["chat"], container_id: str, **params: Any
    ):
        return await self.call_api(
            "im/v1/messages",
            method="GET",
            params={
                "container_id_type": container_id_type,
                "container_id": container_id,
                **params,
            },
        )

    async def get_msg_resource(
        self, message_id: str, file_key: str, type_: Literal["image", "file"]
    ):
        return await self.call_api(
            f"im/v1/messages/{message_id}/resources/{file_key}",
            method="GET",
            params={"type": type_},
        )

    async def get_msg(self, message_id: str):
        return await self.call_api(
            f"im/v1/messages/{message_id}",
            method="GET",
        )

    async def get_msg_read_users(
        self,
        message_id: str,
        user_id_type: str,
        page_size: Optional[int] = None,
        page_token: Optional[str] = None,
    ):
        params: Dict[str, Any] = {"user_id_type": user_id_type}
        if page_size:
            params.update({"page_size": page_size})

        if page_token:
            params.update({"page_token": page_token})

        return await self.call_api(
            f"im/v1/messages/{message_id}/read_users", method="GET", params=params
        )

    async def merge_forward_msg(
        self,
        receive_id_type: str,
        receive_id: str,
        message_id_list: List[str],
        uuid: Optional[str] = None,
    ):
        params = {"receive_id_type": receive_id_type}
        if uuid:
            params.update({"uuid": uuid})

        return await self.call_api(
            "im/v1/messages/merge_forward",
            method="POST",
            params=params,
            json={"receive_id": receive_id, "message_id_list": message_id_list},
        )

    async def forward_msg(
        self,
        message_id: str,
        receive_id: str,
        receive_id_type: str,
        uuid: Optional[str] = None,
    ):
        params = {"receive_id_type": receive_id_type}
        if uuid:
            params.update({"uuid": uuid})

        return await self.call_api(
            f"im/v1/messages/{message_id}/forward",
            method="POST",
            params=params,
            json={"receive_id": receive_id},
        )

    async def delete_msg(self, message_id: str):
        return await self.call_api(f"im/v1/messages/{message_id}", method="DELETE")

    async def edit_msg(self, message_id: str, content: str, msg_type: str):
        return await self.call_api(
            f"im/v1/messages/{message_id}",
            method="PUT",
            json={"msg_type": msg_type, "content": content},
        )

    async def reply_msg(
        self, message_id: str, content: str, msg_type: str, uuid: Optional[str] = None
    ):
        json = {
            "content": content,
            "msg_type": msg_type,
        }

        if uuid:
            json.update({"uuid": uuid})

        return await self.call_api(
            f"im/v1/messages/{message_id}/reply",
            method="POST",
            json=json,
        )

    async def send_msg(
        self,
        receive_id_type: Literal["chat_id", "open_id"],
        receive_id: str,
        content: str,
        msg_type: str,
    ):
        return await self.call_api(
            "im/v1/messages",
            method="POST",
            params={"receive_id_type": receive_id_type},
            json={
                "receive_id": receive_id,
                "content": content,
                "msg_type": msg_type,
            },
        )

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs: Any,
    ) -> Any:
        """根据 `event` 向触发事件的主体回复消息。
        参数:
            event: Event 对象
            message: 要发送的消息
            at_sender (bool): 是否 @ 事件主体
            kwargs: 其他参数，可以与 {ref}`nonebot.adapters.feishu.adapter.Adapter.custom_send` 配合使用
        返回:
            API 调用返回数据
        异常:
            ValueError: 缺少 `user_id`, `group_id`
            NetworkError: 网络错误
            ActionFailed: API 调用失败
        """  # noqa: E501
        return await self.__class__.send_handler(self, event, message, **kwargs)

    @override
    async def call_api(self, api: str, **data) -> Any:
        """
        :说明:
          调用 飞书 协议 API
        :参数:
          * ``api: str``: API 名称
          * ``**data: Any``: API 参数
        :返回:
          - ``Any``: API 调用返回数据
        :异常:
          - ``NetworkError``: 网络错误
          - ``ActionFailed``: API 调用失败
        """
        return await super().call_api(api, **data)

    async def handle_event(self, event: Event) -> None:
        if isinstance(event, MessageEvent):
            _check_at_me(self, event)
            _check_nickname(self, event)
            await _check_reply(self, event)

        await handle_event(self, event)



"""飞书协议适配"""

from .bot import Bot as Bot
from .utils import log as log
from .event import *  # noqa: F403
from .message import PostA as PostA
from .message import PostAt as PostAt
from .adapter import Adapter as Adapter
from .message import Message as Message
from .message import PostImg as PostImg
from .message import PostText as PostText
from .message import PostMedia as PostMedia
from .message import PostEmotion as PostEmotion
from .exception import ActionFailed as ActionFailed
from .exception import NetworkError as NetworkError
from .message import MessageSegment as MessageSegment
from .message import PostMessageNode as PostMessageNode
from .exception import ApiNotAvailable as ApiNotAvailable
from .exception import FeishuAdapterException as FeishuAdapterException
from .message import PostMessageNodeStylable as PostMessageNodeStylable



import base64
import hashlib
from typing import Any, Dict, Optional

from cashews import cache
from Crypto.Cipher import AES
from nonebot.utils import logger_wrapper

from .exception import ActionFailed

log = logger_wrapper("FEISHU")

cache.setup("mem://")


def _handle_api_result(result: Optional[Dict[str, Any]]) -> Any:
    """
    :说明:

      处理 API 请求返回值。

    :参数:

      * ``result: Optional[Dict[str, Any]]``: API 返回数据

    :返回:

        - ``Any``: API 调用返回数据

    :异常:

        - ``ActionFailed``: API 调用失败
    """
    if isinstance(result, dict):
        if result.get("code") != 0:
            raise ActionFailed(**result)
        return result.get("data")
    else:
        return result


class AESCipher:
    def __init__(self, key):
        self.block_size = AES.block_size
        self.key = hashlib.sha256(AESCipher.str_to_bytes(key)).digest()

    @staticmethod
    def str_to_bytes(data):
        u_type = type(b"".decode("utf8"))
        if isinstance(data, u_type):
            return data.encode("utf8")
        return data

    @staticmethod
    def _unpad(s):
        return s[: -ord(s[len(s) - 1 :])]

    def decrypt(self, enc):
        iv = enc[: AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc[AES.block_size :]))

    def decrypt_string(self, enc):
        enc = base64.b64decode(enc)
        return self.decrypt(enc).decode("utf8")
